#---------------------------------------------------------------------------
#
#   SuPy GUI - Modal Dialog - Generic
#
#---------------------------------------------------------------------------

from properties import overridable_property
from window import Window
from button import Button
from layout import Row

class GModalDialog(Window):

	default_button = overridable_property('default_button', "Default button")
	cancel_button = overridable_property('cancel_button', "Cancel button")
	
	_default_button = None
	_cancel_button = None
	
	def __init__(self, contents = None,
			responses = [("OK", True), ("Cancel", False)],
			default = True, cancel = False,
			spacing = 10,
			**kwds):
		Window.__init__(self, style = 'modal_dialog', resizable = False)
		self._cancel_value = cancel
		width = 0
		y = spacing
		if contents:
			width = contents.width
			contents.pos = (spacing, y)
			self.add(contents)
			y += contents.height + spacing
		if responses:
			buttons = []
			for text, value in responses:
				btn = Button(text, action = ('dismiss', value))
				if value == default:
					self.default_button = btn
				if value == cancel:
					self.cancel_button = btn
				buttons.append(btn)
			controls = Row(buttons, equalize = 'w')
			width = max(width, controls.width)
			controls.pos = (spacing + width - controls.width, y)
			self.add(controls)
			y += controls.height + spacing
		self.size = (width + 2 * spacing, y)
		self.set(**kwds)

	def get_default_button(self):
		return self._default_button
	
	def set_default_button(self, x):
		self._default_button = x

	def get_cancel_button(self):
		return self._cancel_button
	
	def set_cancel_button(self, x):
		self._cancel_button = x

	def close(self):
		if self._running:
			self.dismiss(self._cancel_value)
		else:
			Window.close(self)

	def present(self):
		"""Present the dialog modally. Retains control until dismiss() is called
		and then returns the value passed to it."""
		self._value = None
		self.show()
		self._running = True
		try:
			self._present()
		finally:
			self._running = False
			self.hide()
		return self._value

	def dismiss(self, value):
		"""Causes the current call to present() to return with the given value."""
		self._value = value
		self._dismiss()
